@@ -68,6 +68,8 @@ has_element 'contents' =>
 	xml_nodeName => { "" => "Str", "*" => "PRANG::XMLSchema::Whatever" },
 	xml_nodeName_attr => "nodenames",
 	xmlns => "*",
+	xmlns_attr => "nodenames_ns",
+	xml_min => 0,
 	;
 
 has 'nodenames' =>
@@ -75,9 +77,14 @@ has 'nodenames' =>
 	isa => "ArrayRef[Maybe[Str]]",
 	;
 
+has 'nodenames_ns' =>
+	is => "rw",
+	isa => "ArrayRef[Maybe[Str]]",
+	;
+
 has_attr 'attributes' =>
 	is => "rw",
-	isa => "HashRef[Str]",
+	isa => "HashRef[Str|ArrayRef[Str]]",
 	xmlns => "*",
 	xml_name => "*",
 	xmlns_attr => "attributes_ns",
@@ -86,13 +93,11 @@ has_attr 'attributes' =>
 
 has 'attributes_ns' =>
 	is => "rw",
-	isa => "HashRef[Str]",
+	isa => "HashRef[Str|ArrayRef[Str]]",
 	;
 
 sub xmlns { }   # API FIXME - should be able to specify 'any' namespace
 
-with 'PRANG::Graph::Class';
-
 1;
 
 =head1 NAME
